#ifndef MOO_PREFS_H
#define MOO_PREFS_H

#include <moo/types.h>
#include <QVariant>
#include <QStringList>
#include <QFont>

namespace moo {

class Prefs {
    SettingsType type;

public:
    Prefs(SettingsType type) : type(type) {}

    QVariant value(const QString &key, const QVariant &defaultValue = QVariant()) const;
    void setValue(const QString &key, const QVariant &value, const QVariant &defaultValue = QVariant());
    void unsetValue(const QString &key);

    enum ByteArrayKey {
        PageSetup           = 0x001
    };
};

QByteArray prefsValue(Prefs::ByteArrayKey key, const QByteArray &dflt = QByteArray());
void setPrefsValue(Prefs::ByteArrayKey key, const QByteArray &value);

inline QVariant prefsValue(const QString &key, const QVariant &dflt = QVariant())
{
    return Prefs(SettingsPrefs).value(key, dflt);
}

inline QVariant stateValue(const QString &key, const QVariant &dflt = QVariant())
{
    return Prefs(SettingsState).value(key, dflt);
}

template<typename K, typename V>
inline void setPrefsValue(const K &key, const V &value)
{
    Prefs(SettingsPrefs).setValue(key, value);
}

template<typename K, typename V>
inline void setStateValue(const K &key, const V &value)
{
    Prefs(SettingsState).setValue(key, value);
}

template<typename K>
inline void unsetPrefsValue(const K &key)
{
    Prefs(SettingsPrefs).unsetValue(key);
}

template<typename K>
inline void unsetStateValue(const K &key)
{
    Prefs(SettingsState).unsetValue(key);
}

}

#endif // MOO_PREFS_H
